pre_deployments_{{ role }}: {% if not pre_deployments %} [] {% endif %}

{% for deployment in pre_deployments %}
  - {{ deployment }}
{% endfor %}

post_deployments_{{ role }}: {% if not post_deployments %} [] {% endif %}

{% for deployment in post_deployments %}
  - {{ deployment }}
{% endfor %}

{% if ansible_host_vars %}
# ansible_host_vars managed by TripleO Heat Templates in puppet/role.role.j2.yaml:
{{ ansible_host_vars }}
{% endif %}
