container_images_template:

- imagename: "{{namespace}}/{{name_prefix}}aodh-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerAodhApiImage
  - ContainerAodhConfigImage
  services:
  - OS::TripleO::Services::AodhApi
  - OS::TripleO::Services::AodhEvaluator
  - OS::TripleO::Services::AodhListener
  - OS::TripleO::Services::AodhNotifier
  - OS::TripleO::Services::UndercloudAodhApi
  - OS::TripleO::Services::UndercloudAodhEvaluator
  - OS::TripleO::Services::UndercloudAodhListener
  - OS::TripleO::Services::UndercloudAodhNotifier

- imagename: "{{namespace}}/{{name_prefix}}aodh-evaluator{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerAodhEvaluatorImage
  services:
  - OS::TripleO::Services::AodhEvaluator
  - OS::TripleO::Services::UndercloudAodhEvaluator

- imagename: "{{namespace}}/{{name_prefix}}aodh-listener{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerAodhListenerImage
  services:
  - OS::TripleO::Services::AodhListener
  - OS::TripleO::Services::UndercloudAodhListener

- imagename: "{{namespace}}/{{name_prefix}}aodh-notifier{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerAodhNotifierImage
  services:
  - OS::TripleO::Services::AodhNotifier
  - OS::TripleO::Services::UndercloudAodhNotifier

- imagename: "{{namespace}}/{{name_prefix}}barbican-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerBarbicanApiImage
  - ContainerBarbicanConfigImage
  services:
  - OS::TripleO::Services::BarbicanApi

- imagename: "{{namespace}}/{{name_prefix}}barbican-keystone-listener{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerBarbicanKeystoneListenerImage
  - ContainerBarbicanKeystoneListenerConfigImage
  services:
  - OS::TripleO::Services::BarbicanApi
  - OS::TripleO::Services::BarbicanKeystoneListener

- imagename: "{{namespace}}/{{name_prefix}}barbican-worker{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerBarbicanWorkerImage
  - ContainerBarbicanWorkerConfigImage
  services:
  - OS::TripleO::Services::BarbicanApi
  - OS::TripleO::Services::BarbicanWorker

- imagename: "{{namespace}}/{{name_prefix}}ceilometer-central{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCeilometerCentralImage
  - ContainerCeilometerConfigImage
  services:
  - OS::TripleO::Services::CeilometerAgentCentral
  - OS::TripleO::Services::CeilometerAgentIpmi
  - OS::TripleO::Services::CeilometerAgentNotification
  - OS::TripleO::Services::ComputeCeilometerAgent
  - OS::TripleO::Services::UndercloudCeilometerAgentCentral
  - OS::TripleO::Services::UndercloudCeilometerAgentIpmi
  - OS::TripleO::Services::UndercloudCeilometerAgentNotification

- imagename: "{{namespace}}/{{name_prefix}}ceilometer-compute{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCeilometerComputeImage
  services:
  - OS::TripleO::Services::ComputeCeilometerAgent

- imagename: "{{namespace}}/{{name_prefix}}ceilometer-notification{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCeilometerNotificationImage
  services:
  - OS::TripleO::Services::CeilometerAgentNotification
  - OS::TripleO::Services::UndercloudCeilometerAgentNotification

- imagename: "{{namespace}}/{{name_prefix}}ceilometer-ipmi{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCeilometerIpmiImage
  services:
  - OS::TripleO::Services::CeilometerAgentIpmi
  - OS::TripleO::Services::UndercloudCeilometerAgentIpmi

- imagename: "{{namespace}}/{{name_prefix}}cinder-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCinderApiImage
  - ContainerCinderConfigImage
  services:
  - OS::TripleO::Services::BlockStorageCinderVolume
  - OS::TripleO::Services::CinderApi
  - OS::TripleO::Services::CinderBackup
  - OS::TripleO::Services::CinderScheduler
  - OS::TripleO::Services::CinderVolume
  - OS::TripleO::Services::UndercloudCinderApi
  - OS::TripleO::Services::UndercloudCinderScheduler
  - OS::TripleO::Services::UndercloudCinderVolume

- imagename: "{{namespace}}/{{name_prefix}}cinder-backup{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCinderBackupImage
  services:
  - OS::TripleO::Services::CinderBackup

- imagename: "{{namespace}}/{{name_prefix}}cinder-scheduler{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCinderSchedulerImage
  services:
  - OS::TripleO::Services::CinderScheduler
  - OS::TripleO::Services::UndercloudCinderScheduler

- imagename: "{{namespace}}/{{name_prefix}}cinder-volume{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCinderVolumeImage
  services:
  - OS::TripleO::Services::BlockStorageCinderVolume
  - OS::TripleO::Services::CinderVolume
  - OS::TripleO::Services::UndercloudCinderVolume

- imagename: "{{namespace}}/{{name_prefix}}collectd{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCollectdConfigImage
  - ContainerCollectdImage
  services:
  - OS::TripleO::Services::Collectd

- imagename: "{{namespace}}/{{name_prefix}}cron{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerCrondConfigImage
  - ContainerCrondImage
  services:
  - OS::TripleO::Services::ContainersLogrotateCrond

- imagename: "{{namespace}}/{{name_prefix}}designate-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerDesignateApiImage
  services:
  - OS::TripleO::Services::DesignateApi
  - OS::TripleO::Services::DesignateCentral
  - OS::TripleO::Services::DesignateMDNS
  - OS::TripleO::Services::DesignateSink
  - OS::TripleO::Services::DesignateProducer
  - OS::TripleO::Services::DesignateWorker

- imagename: "{{namespace}}/{{name_prefix}}designate-backend-bind9{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerDesignateBackendBIND9Image
  services:
  - OS::TripleO::Services::DesignateWorker

- imagename: "{{namespace}}/{{name_prefix}}designate-central{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerDesignateCentralImage
  services:
  - OS::TripleO::Services::DesignateCentral

- imagename: "{{namespace}}/{{name_prefix}}designate-mdns{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerDesignateMDNSImage
  services:
  - OS::TripleO::Services::DesignateMDNS

- imagename: "{{namespace}}/{{name_prefix}}designate-producer{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerDesignateProducerImage
  services:
  - OS::TripleO::Services::DesignateProducer

- imagename: "{{namespace}}/{{name_prefix}}designate-sink{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerDesignateSinkImage
  services:
  - OS::TripleO::Services::DesignateSink

- imagename: "{{namespace}}/{{name_prefix}}designate-worker{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerDesignateConfigImage
  - ContainerDesignateWorkerImage
  services:
  - OS::TripleO::Services::DesignateWorker

- imagename: "{{namespace}}/{{name_prefix}}etcd{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerEtcdConfigImage
  - ContainerEtcdImage
  services:
  - OS::TripleO::Services::Etcd

- imagename: "{{namespace}}/{{name_prefix}}glance-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerGlanceApiConfigImage
  - ContainerGlanceApiImage
  services:
  - OS::TripleO::Services::GlanceApi
  - OS::TripleO::Services::GlanceApiEdge

- imagename: "{{namespace}}/{{name_prefix}}gnocchi-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerGnocchiApiImage
  - ContainerGnocchiConfigImage
  services:
  - OS::TripleO::Services::GnocchiApi
  - OS::TripleO::Services::GnocchiMetricd
  - OS::TripleO::Services::GnocchiStatsd
  - OS::TripleO::Services::UndercloudGnocchiApi
  - OS::TripleO::Services::UndercloudGnocchiMetricd
  - OS::TripleO::Services::UndercloudGnocchiStatsd

- imagename: "{{namespace}}/{{name_prefix}}gnocchi-metricd{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerGnocchiMetricdImage
  services:
  - OS::TripleO::Services::GnocchiMetricd
  - OS::TripleO::Services::UndercloudGnocchiMetricd

- imagename: "{{namespace}}/{{name_prefix}}gnocchi-statsd{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerGnocchiStatsdImage
  services:
  - OS::TripleO::Services::GnocchiStatsd
  - OS::TripleO::Services::UndercloudGnocchiStatsd

- imagename: "{{namespace}}/{{name_prefix}}haproxy{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerHAProxyConfigImage
  - ContainerHAProxyImage
  services:
  - OS::TripleO::Services::HAproxy
  - OS::TripleO::Services::HAproxyEdge
  - OS::TripleO::Services::UndercloudHAProxy

- imagename: "{{namespace}}/{{name_prefix}}heat-all{{name_suffix}}:{{tag}}"
  image_source: kolla

- imagename: "{{namespace}}/{{name_prefix}}heat-api-cfn{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerHeatApiCfnConfigImage
  - ContainerHeatApiCfnImage
  services:
  - OS::TripleO::Services::HeatApiCfn

- imagename: "{{namespace}}/{{name_prefix}}heat-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerHeatApiConfigImage
  - ContainerHeatApiImage
  - ContainerHeatConfigImage
  services:
  - OS::TripleO::Services::HeatApi
  - OS::TripleO::Services::HeatEngine

- imagename: "{{namespace}}/{{name_prefix}}heat-engine{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerHeatEngineImage
  services:
  - OS::TripleO::Services::HeatEngine

- imagename: "{{namespace}}/{{name_prefix}}horizon{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerHorizonConfigImage
  - ContainerHorizonImage
  services:
  - OS::TripleO::Services::Horizon

- imagename: "{{namespace}}/{{name_prefix}}ironic-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerIronicApiImage
  - ContainerIronicApiConfigImage
  services:
  - OS::TripleO::Services::IronicApi

- imagename: "{{namespace}}/{{name_prefix}}ironic-conductor{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerIronicConductorImage
  services:
  - OS::TripleO::Services::IronicConductor

- imagename: "{{namespace}}/{{name_prefix}}ironic-inspector{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerIronicInspectorImage
  - ContainerIronicInspectorConfigImage
  services:
  - OS::TripleO::Services::IronicInspector

- imagename: "{{namespace}}/{{name_prefix}}ironic-pxe{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerIronicConfigImage
  - ContainerIronicPxeImage
  services:
  - OS::TripleO::Services::IronicConductor
  - OS::TripleO::Services::IronicPxe

- imagename: "{{namespace}}/{{name_prefix}}ironic-neutron-agent{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerIronicNeutronAgentImage
  - ContainerNeutronConfigImage
  services:
  - OS::TripleO::Services::IronicNeutronAgent

- imagename: "{{namespace}}/{{name_prefix}}iscsid{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerIscsidConfigImage
  - ContainerIscsidImage
  services:
  - OS::TripleO::Services::Iscsid

- imagename: "{{namespace}}/{{name_prefix}}keepalived{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerKeepalivedConfigImage
  - ContainerKeepalivedImage
  services:
  - OS::TripleO::Services::Keepalived
  - OS::TripleO::Services::UndercloudKeepalived

- imagename: "{{namespace}}/{{name_prefix}}keystone{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerKeystoneConfigImage
  - ContainerKeystoneImage
  services:
  - OS::TripleO::Services::Keystone

- imagename: "{{namespace}}/{{name_prefix}}manila-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerManilaApiImage
  - ContainerManilaConfigImage
  services:
  - OS::TripleO::Services::ManilaApi
  - OS::TripleO::Services::ManilaScheduler
  - OS::TripleO::Services::ManilaShare

- imagename: "{{namespace}}/{{name_prefix}}manila-scheduler{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerManilaSchedulerImage
  services:
  - OS::TripleO::Services::ManilaScheduler

- imagename: "{{namespace}}/{{name_prefix}}manila-share{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerManilaShareImage
  services:
  - OS::TripleO::Services::ManilaShare

- imagename: "{{namespace}}/{{name_prefix}}mariadb{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerClustercheckConfigImage
  - ContainerClustercheckImage
  - ContainerMysqlConfigImage
  - ContainerMysqlImage
  - ContainerMysqlClientConfigImage
  services:
  - OS::TripleO::Services::MySQL

- imagename: "{{namespace}}/{{name_prefix}}memcached{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerMemcachedConfigImage
  - ContainerMemcachedImage
  services:
  - OS::TripleO::Services::Memcached

- imagename: "{{namespace}}/{{name_prefix}}mistral-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerMistralApiImage
  - ContainerMistralConfigImage
  services:
  - OS::TripleO::Services::MistralApi
  - OS::TripleO::Services::MistralEngine
  - OS::TripleO::Services::MistralExecutor
  - OS::TripleO::Services::MistralEventEngine

- imagename: "{{namespace}}/{{name_prefix}}mistral-engine{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerMistralEngineImage
  services:
  - OS::TripleO::Services::MistralEngine

- imagename: "{{namespace}}/{{name_prefix}}mistral-executor{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerMistralExecutorImage
  services:
  - OS::TripleO::Services::MistralExecutor

- imagename: "{{namespace}}/{{name_prefix}}mistral-event-engine{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerMistralEventEngineImage
  services:
  - OS::TripleO::Services::MistralEventEngine

- imagename: "{{namespace}}/{{name_prefix}}multipathd{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerMultipathdConfigImage
  - ContainerMultipathdImage
  services:
  - OS::TripleO::Services::Multipathd

- imagename: "{{namespace}}/{{name_prefix}}neutron-dhcp-agent{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNeutronDHCPImage
  services:
  - OS::TripleO::Services::NeutronDhcpAgent

- imagename: "{{namespace}}/{{name_prefix}}neutron-l3-agent{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNeutronL3AgentImage
  services:
  - OS::TripleO::Services::NeutronL3Agent

- imagename: "{{namespace}}/{{name_prefix}}neutron-metadata-agent{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNeutronMetadataImage
  services:
  - OS::TripleO::Services::NeutronMetadataAgent

- imagename: "{{namespace}}/{{name_prefix}}neutron-openvswitch-agent{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOpenvswitchImage
  services:
  - OS::TripleO::Services::ComputeNeutronOvsAgent
  - OS::TripleO::Services::NeutronOvsAgent

- imagename: "{{namespace}}/{{name_prefix}}neutron-sriov-agent{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNeutronSriovImage
  services:
  - OS::TripleO::Services::NeutronSriovAgent

- imagename: "{{namespace}}/{{name_prefix}}neutron-mlnx-agent{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNeutronMlnxImage
  services:
  - OS::TripleO::Services::NeutronMlnxAgent

- imagename: "{{namespace}}/{{name_prefix}}neutron-server{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNeutronApiImage
  - ContainerNeutronConfigImage
  services:
  - OS::TripleO::Services::ComputeNeutronOvsAgent
  - OS::TripleO::Services::NeutronApi
  - OS::TripleO::Services::NeutronCorePlugin
  - OS::TripleO::Services::NeutronDhcpAgent
  - OS::TripleO::Services::NeutronL3Agent
  - OS::TripleO::Services::NeutronMetadataAgent
  - OS::TripleO::Services::NeutronOvsAgent
  - OS::TripleO::Services::NeutronServer
  - OS::TripleO::Services::NeutronSriovAgent
{% if neutron_driver == "ovn" %}
  - OS::TripleO::Services::OVNController

- imagename: "{{namespace}}/{{name_prefix}}neutron-metadata-agent-ovn{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOvnMetadataImage
  services:
  - OS::TripleO::Services::OVNMetadataAgent
{% endif %}

- imagename: "{{namespace}}/{{name_prefix}}nova-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovaApiImage
  - ContainerNovaConfigImage
  - ContainerNovaMetadataImage
  - ContainerNovaMetadataConfigImage
  services:
  - OS::TripleO::Services::NovaApi
  - OS::TripleO::Services::NovaConductor
  - OS::TripleO::Services::NovaIronic
  - OS::TripleO::Services::NovaScheduler
  - OS::TripleO::Services::NovaVncProxy
  - OS::TripleO::Services::NovaMetadata

- imagename: "{{namespace}}/{{name_prefix}}nova-compute-ironic{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovaComputeIronicImage
  services:
  - OS::TripleO::Services::NovaIronic

- imagename: "{{namespace}}/{{name_prefix}}nova-compute{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovaComputeImage
  - ContainerNovaLibvirtConfigImage
  services:
  - OS::TripleO::Services::NovaCompute
  - OS::TripleO::Services::NovaLibvirt
  - OS::TripleO::Services::NovaMigrationTarget

- imagename: "{{namespace}}/{{name_prefix}}nova-conductor{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovaConductorImage
  services:
  - OS::TripleO::Services::NovaConductor

- imagename: "{{namespace}}/{{name_prefix}}nova-libvirt{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovaLibvirtImage
  services:
  - OS::TripleO::Services::NovaLibvirt

- imagename: "{{namespace}}/{{name_prefix}}nova-novncproxy{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovaVncProxyImage
  services:
  - OS::TripleO::Services::NovaVncProxy

- imagename: "{{namespace}}/{{name_prefix}}nova-scheduler{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovaSchedulerImage
  services:
  - OS::TripleO::Services::NovaScheduler

- imagename: "{{namespace}}/{{name_prefix}}novajoin-notifier{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovajoinNotifierImage
  services:
  - OS::TripleO::Services::Novajoin

- imagename: "{{namespace}}/{{name_prefix}}novajoin-server{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerNovajoinServerImage
  - ContainerNovajoinConfigImage
  services:
  - OS::TripleO::Services::Novajoin

- imagename: "{{namespace}}/{{name_prefix}}octavia-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOctaviaApiImage
  - ContainerOctaviaConfigImage
  - ContainerOctaviaDriverAgentImage
  - ContainerOctaviaDriverAgentConfigImage
  services:
  - OS::TripleO::Services::OctaviaApi
  - OS::TripleO::Services::OctaviaHealthManager
  - OS::TripleO::Services::OctaviaHousekeeping
  - OS::TripleO::Services::OctaviaWorker

- imagename: "{{namespace}}/{{name_prefix}}octavia-health-manager{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOctaviaHealthManagerImage
  services:
  - OS::TripleO::Services::OctaviaHealthManager

- imagename: "{{namespace}}/{{name_prefix}}octavia-housekeeping{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOctaviaHousekeepingImage
  services:
  - OS::TripleO::Services::OctaviaHousekeeping

- imagename: "{{namespace}}/{{name_prefix}}octavia-worker{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOctaviaWorkerImage
  services:
  - OS::TripleO::Services::OctaviaWorker

{% if neutron_driver == "ovn" %}
- imagename: "{{namespace}}/{{name_prefix}}ovn-controller{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOvnControllerConfigImage
  - ContainerOvnControllerImage
  services:
  - OS::TripleO::Services::OVNController

- imagename: "{{namespace}}/{{name_prefix}}ovn-nb-db-server{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOvnNbDbImage
  services:
  - OS::TripleO::Services::OVNDBs

- imagename: "{{namespace}}/{{name_prefix}}ovn-northd{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOvnNorthdImage
  - ContainerOvnDbsImage
  - ContainerOvnDbsConfigImage
  services:
  - OS::TripleO::Services::OVNDBs

- imagename: "{{namespace}}/{{name_prefix}}ovn-sb-db-server{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerOvnSbDbImage
  services:
  - OS::TripleO::Services::OVNDBs
{% endif %}

- imagename: "{{namespace}}/{{name_prefix}}placement-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerPlacementConfigImage
  - ContainerPlacementImage
  services:
  - OS::TripleO::Services::PlacementApi

- imagename: "{{namespace}}/{{name_prefix}}qdrouterd{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerQdrouterdConfigImage
  - ContainerQdrouterdImage
  - ContainerMetricsQdrConfigImage
  - ContainerMetricsQdrImage
  services:
  - OS::TripleO::Services::Qdr
  - OS::TripleO::Services::OsloMessagingRpc
  - OS::TripleO::Services::MetricsQdr

- imagename: "{{namespace}}/{{name_prefix}}rabbitmq{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerRabbitmqConfigImage
  - ContainerRabbitmqImage
  services:
  - OS::TripleO::Services::RabbitMQ
  - OS::TripleO::Services::OsloMessagingRpc
  - OS::TripleO::Services::OsloMessagingNotify

- imagename: "{{namespace}}/{{name_prefix}}redis{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerRedisConfigImage
  - ContainerRedisImage
  services:
  - OS::TripleO::Services::Redis
  - OS::TripleO::Services::UndercloudRedis

- imagename: "{{namespace}}/{{name_prefix}}sahara-api{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerSaharaApiImage
  - ContainerSaharaConfigImage
  services:
  - OS::TripleO::Services::SaharaApi
  - OS::TripleO::Services::SaharaEngine

- imagename: "{{namespace}}/{{name_prefix}}sahara-engine{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerSaharaEngineImage
  services:
  - OS::TripleO::Services::SaharaEngine

- imagename: "{{namespace}}/{{name_prefix}}swift-account{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerSwiftAccountImage
  services:
  - OS::TripleO::Services::SwiftStorage

- imagename: "{{namespace}}/{{name_prefix}}swift-container{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerSwiftContainerImage
  services:
  - OS::TripleO::Services::SwiftStorage

- imagename: "{{namespace}}/{{name_prefix}}swift-object{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerSwiftObjectImage
  services:
  - OS::TripleO::Services::SwiftStorage

- imagename: "{{namespace}}/{{name_prefix}}swift-proxy-server{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerSwiftConfigImage
  - ContainerSwiftProxyImage
  services:
  - OS::TripleO::Services::SwiftProxy
  - OS::TripleO::Services::SwiftRingBuilder
  - OS::TripleO::Services::SwiftStorage

# NOTE: the TripleoClient service will likely be just a stub to allow
# us to obtain this container via the prepare commands.
- imagename: "{{namespace}}/{{name_prefix}}tripleoclient{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerTripleoclientImage
  services:
  - OS::TripleO::Services::TripleoClient

- imagename: "{{namespace}}/{{name_prefix}}zaqar-wsgi{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerZaqarConfigImage
  - ContainerZaqarImage
  services:
  - OS::TripleO::Services::Zaqar

- imagename: "{{ceph_namespace}}/{{ceph_image}}:{{ceph_tag}}"
  image_source: ceph
  params:
  - ContainerCephDaemonImage
  services:
  - OS::TripleO::Services::CephClient
  - OS::TripleO::Services::CephExternal
  - OS::TripleO::Services::CephMds
  - OS::TripleO::Services::CephMgr
  - OS::TripleO::Services::CephMon
  - OS::TripleO::Services::CephOSD
  - OS::TripleO::Services::CephRgw
  - OS::TripleO::Services::CephRbdMirror

- imagename: "{{ceph_prometheus_namespace}}/{{ceph_prometheus_image}}:{{ceph_prometheus_tag}}"
  image_source: prom
  params:
  - PrometheusContainerImage
  services:
  - OS::TripleO::Services::CephGrafana

- imagename: "{{ceph_alertmanager_namespace}}/{{ceph_alertmanager_image}}:{{ceph_alertmanager_tag}}"
  image_source: prom
  params:
  - AlertManagerContainerImage
  services:
  - OS::TripleO::Services::CephGrafana

- imagename: "{{ceph_node_exporter_namespace}}/{{ceph_node_exporter_image}}:{{ceph_node_exporter_tag}}"
  image_source: prom
  params:
  - NodeExporterContainerImage
  services:
  - OS::TripleO::Services::CephGrafana

- imagename: "{{ceph_grafana_namespace}}/{{ceph_grafana_image}}:{{ceph_grafana_tag}}"
  image_source: grafana
  params:
  - GrafanaContainerImage
  services:
  - OS::TripleO::Services::CephGrafana

- imagename: "{{namespace}}/{{name_prefix}}tempest{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerTempestImage
  services:
  - OS::TripleO::Services::Tempest

- imagename: "{{namespace}}/{{name_prefix}}rsyslog{{name_suffix}}:{{tag}}"
  image_source: kolla
  params:
  - ContainerRsyslogImage
  - ContainerRsyslogConfigImage
  - ContainerRsyslogSidecarImage
  - ContainerRsyslogSidecarConfigImage
  services:
  - OS::TripleO::Services::HAproxy
  - OS::TripleO::Services::Rsyslog
  - OS::TripleO::Services::SwiftProxy
  - OS::TripleO::Services::SwiftStorage

{% if ceph3_namespace is defined %}
- imagename: "{{ceph3_namespace}}/{{ceph3_image}}:{{ceph3_tag}}"
  image_source: ceph
  params:
  - ContainerCeph3DaemonImage
  services:
  - OS::TripleO::Services::CephClient
  - OS::TripleO::Services::CephExternal
  - OS::TripleO::Services::CephMds
  - OS::TripleO::Services::CephMgr
  - OS::TripleO::Services::CephMon
  - OS::TripleO::Services::CephOSD
  - OS::TripleO::Services::CephRgw
  - OS::TripleO::Services::CephRbdMirror
{% endif %}

{% if namespace_stein is defined %}
- imagename: "{{namespace_stein}}/{{name_prefix_stein}}cinder-api{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerCinderApiImageStein
  services:
  - OS::TripleO::Services::CinderApi
  - OS::TripleO::Services::UndercloudCinderApi

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}glance-api{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerGlanceApiImageStein
  services:
  - OS::TripleO::Services::GlanceApi
  - OS::TripleO::Services::GlanceApiEdge

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}heat-engine{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerHeatEngineImageStein
  services:
  - OS::TripleO::Services::HeatEngine

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}ironic-api{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerIronicApiImageStein
  services:
  - OS::TripleO::Services::IronicApi

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}ironic-inspector{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerIronicInspectorImageStein
  services:
  - OS::TripleO::Services::IronicInspector

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}keystone{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerKeystoneImageStein
  services:
  - OS::TripleO::Services::Keystone

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}mistral-api{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerMistralApiImageStein
  services:
  - OS::TripleO::Services::MistralApi

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}neutron-server{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerNeutronApiImageStein
  services:
  - OS::TripleO::Services::NeutronApi

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}nova-api{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerNovaApiImageStein
  services:
  - OS::TripleO::Services::NovaApi

- imagename: "{{namespace_stein}}/{{name_prefix_stein}}nova-conductor{{name_suffix_stein}}:{{tag_stein}}"
  image_source: kolla
  params:
  - ContainerNovaConductorImageStein
  services:
  - OS::TripleO::Services::NovaConductor
{% endif %}
